@@ -7,6 +7,7 @@
 @homepage: https://gitlab.com/Malfatti/SciScripts
 """
 
+print('[Analysis.Analysis] Importing dependencies...')
 import numpy as np
 import os
 import scipy.signal as ssig
@@ -16,6 +17,7 @@ from scipy.stats.stats import pearsonr
 from scipy import fftpack
 
 from sciscripts.IO import IO
+print('[Analysis.Analysis] Done.')
 
 
 ## Level 0
@@ -199,23 +201,6 @@ def GetFxx(Rate, NFFT, FreqWindow):
     return(Fxx)
 
 
-def GetPhase(Signal):
-    if len(Signal.shape) > 1:
-        Data = np.zeros(Signal.shape)
-        for C in range(Data.shape[1]):
-            print(f'Processing ch {C+1}...')
-            Data[:,C] = GetPhase(Signal[:,C])
-
-    else:
-        # Mean = Signal.mean(axis=0)
-        # Orders faster when signal size is a power of two, so padding and truncating
-        Data = ssig.hilbert(Signal, fftpack.next_fast_len(len(Signal)))[:len(Signal)]
-        Data = np.angle(Data)
-        # Data += Mean
-
-    return(Data)
-
-
 def GetAmpEnv(Signal):
     if len(Signal.shape) > 1:
         Data = np.zeros(Signal.shape, 'float32')
@@ -233,24 +218,6 @@ def GetAmpEnv(Signal):
     return(Data)
 
 
-def GetDeltaPhase(SignalRef, SignalResult):
-    if len(SignalRef.shape) > 1:
-        if SignalRef.shape[1] != SignalResult.shape[1]:
-            print('Ref and Result dimensions have to be the same.')
-            return(None)
-
-        DeltaPhase = np.zeros(SignalRef.shape)
-        for C in range(DeltaPhase.shape[1]):
-            DeltaPhase[:,C] = GetDeltaPhase(SignalRef[:,C], SignalResult[:,C])
-
-    else:
-        PhaseRef = GetPhase(SignalRef)
-        PhaseResult = GetPhase(SignalResult)
-        DeltaPhase = np.angle(np.exp(1j * (PhaseRef - PhaseResult)))
-
-    return(DeltaPhase)
-
-
 def GetEventEdges(EventsDict, Ch, Proc, Rec, Edge='rise'):
     if Edge.lower() == 'rise': Id = 1
     elif Edge.lower() == 'fall': Id = 0
@@ -326,6 +293,23 @@ def GetPeaks(Signal, Std=1, FixedThreshold=None):
     return(Peaks)
 
 
+def GetPhase(Signal):
+    if len(Signal.shape) > 1:
+        Data = np.zeros(Signal.shape)
+        for C in range(Data.shape[1]):
+            print(f'Processing ch {C+1}...')
+            Data[:,C] = GetPhase(Signal[:,C])
+
+    else:
+        # Mean = Signal.mean(axis=0)
+        # Orders faster when signal size is a power of two, so padding and truncating
+        Data = ssig.hilbert(Signal, fftpack.next_fast_len(len(Signal)))[:len(Signal)]
+        Data = np.angle(Data)
+        # Data += Mean
+
+    return(Data)
+
+
 def GetPowerOf2(N):
     Next = int(np.ceil(np.log2(N)))
     return(Next)
@@ -341,12 +325,6 @@ def GetRecXValues(TTLs, Rate, TimeBeforeTTL, TimeAfterTTL):
     return(XValues)
 
 
-def GetStrongestCh(Data):
-    BestCh = [np.mean(Data[:,Ch]**2)**0.5 for Ch in range(Data.shape[1])]
-    BestCh = BestCh.index(max(BestCh))
-    return(BestCh)
-
-
 def GetTTLInfo(Events, EventRec, TTLCh):
     print('Get TTL data...')
     EventID = Events['TTLs']['user_data']['eventID']
@@ -509,6 +487,16 @@ def RemapChannels(Tip, Head, Connector):
     return(ChMap)
 
 
+def RMS(Data):
+    if len(Data.shape) == 2:
+        DataRMS = np.empty(Data.shape[1])
+        for C in range(Data.shape[1]): DataRMS[C] = RMS(Data[:,C])
+    else:
+        DataRMS = np.mean(Data**2)**0.5
+
+    return(DataRMS)
+
+
 def StrRange(Start='a', End='z', Step=1):
     if max(len(Start), len(End)) > 1:
         print('Only 1-char length strings are accepted.')
@@ -531,6 +519,70 @@ def SubSample(Data, Rate, NewRate, t=[]):
     else: return(Data)
 
 
+def Upsample2d(Data, Dim1, Dim2, Up):
+    """
+    Upsample a 2-dimensional array.
+
+    Parameters
+    ----------
+    Data: 2d array
+        Array of shape (Dim1.shape[0], Dim2.shape[0]) containing the values.
+
+    Dim1: 1d array
+        Array of shape (Data.shape[0],) representing the 1st dimension.
+
+    Dim2: 1d array
+        Array of shape (Data.shape[1],) representing the 2nd dimension.
+
+    Up: int, list
+        How many times should the dimensions be upsampled. If int, the value is
+        applied for both dimensions. If list, the values are applied to the
+        dimensions, respectively.
+
+    Returns
+    -------
+    DataUp: 2d array
+        The upsampled array.
+
+    Dim1Up: 1d array
+        The upsampled array for the 1st dimension.
+
+    Dim2Up: 1d array
+        The upsampled array for the 2nd dimension.
+
+    Example
+    -------
+    To upsample a spectrogram 5x in both dimension:
+        SxxUp, FUp, TUp = SxxUpsample(Sxx, F, T, 5)
+    """
+    DimsUp = [
+        np.interp(
+            np.linspace(0,1,len(Dim)*Up),
+            np.linspace(0,1,len(Dim)),
+            Dim
+        )
+        for Dim in [Dim1, Dim2]
+    ]
+
+    Data1Up = np.zeros((len(Dim1),len(DimsUp[1])), dtype='float32')
+    for I in range(Data.shape[0]):
+        Data1Up[I,:] = np.interp(
+            np.linspace(0,1,Data1Up.shape[1]),
+            np.linspace(0,1,Data.shape[1]),
+            Data[I,:]
+        )
+
+    DataUp = np.zeros((len(DimsUp[0]),len(DimsUp[1])), dtype='float32')
+    for F in range(Data1Up.shape[1]):
+        DataUp[:,F] = np.interp(
+            np.linspace(0,1,len(DimsUp[0])),
+            np.linspace(0,1,Data1Up.shape[0]),
+            Data1Up[:,F]
+        )
+
+    return(DataUp, DimsUp[0], DimsUp[1])
+
+
 def UniqueStr(List, KeepOrder=False):
     """ Better alternative: numpy.unique(List) """
     if KeepOrder:
@@ -680,6 +732,31 @@ def CWT(Signal, t, Rate, Freqs, tPsi, Wavelet=Morlet, WaveletArgs=dict(CyclesNo=
     return(Signal_Psi)
 
 
+def GetDeltaPhase(SignalRef, SignalResult):
+    if len(SignalRef.shape) > 1:
+        if SignalRef.shape[1] != SignalResult.shape[1]:
+            print('Ref and Result dimensions have to be the same.')
+            return(None)
+
+        DeltaPhase = np.zeros(SignalRef.shape)
+        for C in range(DeltaPhase.shape[1]):
+            DeltaPhase[:,C] = GetDeltaPhase(SignalRef[:,C], SignalResult[:,C])
+
+    else:
+        PhaseRef = GetPhase(SignalRef)
+        PhaseResult = GetPhase(SignalResult)
+        DeltaPhase = np.angle(np.exp(1j * (PhaseRef - PhaseResult)))
+
+    return(DeltaPhase)
+
+
+def GetEntropyMI(Hist):
+    H = Entropy(Hist)[0]
+    Probs = Hist/np.sum(Hist)
+    MI = (np.log(Probs.shape[0])-H)/np.log(Probs.shape[0])
+    return(MI)
+
+
 def GetInstFreq(Signal, Rate):
     if len(Signal.shape) == 2:
 
@@ -699,13 +776,6 @@ def GetInstFreq(Signal, Rate):
     return(InstFreq)
 
 
-def GetEntropyMI(Hist):
-    H = Entropy(Hist)[0]
-    Probs = Hist/np.sum(Hist)
-    MI = (np.log(Probs.shape[0])-H)/np.log(Probs.shape[0])
-    return(MI)
-
-
 def GetNFFT(WindowSize, Pad=0):
     NFFT = int(2**(GetPowerOf2(WindowSize)+Pad))
     return(NFFT)
@@ -732,6 +802,13 @@ def GetPLVs(SignalRef, SignalResult, Rate, Freqs, FreqBand):
     return(PLVs)
 
 
+def GetStrongestCh(Data):
+    # BestCh = [np.mean(Data[:,Ch]**2)**0.5 for Ch in range(Data.shape[1])]
+    # BestCh = BestCh.index(max(BestCh))
+    BestCh = RMS(Data).argmax()
+    return(BestCh)
+
+
 def PSD(Signal, Rate, Scaling='density', WindowSize=None, NPerSeg=None, NOverlap=None):
     if not WindowSize: WindowSize = len(Signal)
     if not NPerSeg: NPerSeg = WindowSize//2
@@ -816,14 +893,14 @@ def SignalIntensity(Signal, Rate, FreqBand, Ref, NoiseRMS=None, WindowSize=None)
     Range = (F > FreqBand[0])*(F < FreqBand[1])
     BinSize = F[1] - F[0]
 
-    RMS = (sum(PxxSp[Range]) * BinSize)**0.5
-    if NoiseRMS: RMS = RMS - NoiseRMS
+    PxxRMS = (sum(PxxSp[Range]) * BinSize)**0.5
+    if NoiseRMS: PxxRMS = PxxRMS - NoiseRMS
 
-    dB = 20*(np.log10((RMS/Ref)/2e-5))
+    dB = 20*(np.log10((PxxRMS/Ref)/2e-5))
 
     IntensityPSD['F'] = F
     IntensityPSD['PxxSp'] = PxxSp
-    Intensity['RMS'] = RMS
+    Intensity['RMS'] = PxxRMS
     Intensity['dB'] = dB
 
     return(Intensity, IntensityPSD)
