@@ -10,7 +10,7 @@
 import numpy as np
 from scipy import signal
 
-from sciscripts.Analysis import Analysis
+from sciscripts.Analysis import Analysis as sAnalysis
 from sciscripts.IO import IO#, Asdf#, Hdf5
 
 
@@ -98,7 +98,7 @@ def GetExpsIndexesDict(Exps, ExpNames=None):
             #                                     for K, V in GPIASRec['Index'].items()}}
 
             Indexes_IFAE[E][Animal] = {**Indexes_IFAE[E][Animal],
-                                             **{K: Analysis.GetNegEquiv(V['GPIASIndex'])
+                                             **{K: sAnalysis.GetNegEquiv(V['GPIASIndex'])
                                                 for K, V in GPIASRec['Index'].items()}}
             Traces_IFAE[E][Animal] = {**Traces_IFAE[E][Animal], **GPIASRec['Trace']}
 
@@ -132,7 +132,7 @@ def GetExpsIndexesDictBest(Exps, ExpNames=None):
             #                                  **{K: np.abs(V['GPIASIndex'])
             #                                     for K, V in GPIASRec['Index'].items()}}
 
-            Dict = {K: Analysis.GetNegEquiv(V['GPIASIndex']) for K, V in GPIASRec['Index'].items()}
+            Dict = {K: sAnalysis.GetNegEquiv(V['GPIASIndex']) for K, V in GPIASRec['Index'].items()}
             for K in Dict:
                 if K in Indexes_IFAE[E][Animal]:
                     if Dict[K] < Indexes_IFAE[E][Animal][K]:
@@ -156,11 +156,39 @@ def GetMAF(Indexes_IFAE, Animals, ExpOrder):
     for Animal in Animals:
         ## Indexes_IFAE as array
         ThisAnimal = Indexes_IFAE['Animals'] == Animal
-        Freqs = [Indexes_IFAE['Freqs'][ThisAnimal*(Indexes_IFAE['Exps'] == Exp)] for Exp in ExpOrder[:2]]
-        Freqs = sorted(np.intersect1d(Freqs[0], Freqs[1]), key=lambda x: int(x.split('-')[-1]))
-        MAF_A = [abs(Indexes_IFAE['Index'][ThisAnimal*(Indexes_IFAE['Exps'] == ExpOrder[1])*(Indexes_IFAE['Freqs'] == Freq)]) -
-                  abs(Indexes_IFAE['Index'][ThisAnimal*(Indexes_IFAE['Exps'] == ExpOrder[0])*(Indexes_IFAE['Freqs'] == Freq)])
-                  for Freq in Freqs]
+
+        Freqs = [
+            Indexes_IFAE['Freqs'][ThisAnimal*(Indexes_IFAE['Exps'] == Exp)]
+            for Exp in ExpOrder[:2]
+        ]
+
+        FreqsI = np.array([
+            abs(Indexes_IFAE['Index'][
+                ThisAnimal*(Indexes_IFAE['Exps'] == ExpOrder[0])
+                *(Indexes_IFAE['Freqs'] == Freq)
+            ])
+            for Freq in Freqs[0]
+        ]).ravel()
+
+        Freqs[0] = np.array(Freqs[0])[FreqsI > 30].tolist()
+        Freqs = sorted(
+            np.intersect1d(Freqs[0], Freqs[1]),
+            key=lambda x: int(x.split('-')[-1])
+        )
+
+        ExpFIDiff = [
+            abs(Indexes_IFAE['Index'][
+                ThisAnimal*(Indexes_IFAE['Exps'] == ExpOrder[1])
+                *(Indexes_IFAE['Freqs'] == Freq)
+            ])
+            - abs(Indexes_IFAE['Index'][
+                ThisAnimal*(Indexes_IFAE['Exps'] == ExpOrder[0])
+                *(Indexes_IFAE['Freqs'] == Freq)
+            ])
+            for Freq in Freqs
+        ]
+
+
 
         ## Indexes_IFAE as dict
         # FreqsB = sorted(Indexes_IFAE[ExpOrder[0]][Animal].keys(),
@@ -171,18 +199,18 @@ def GetMAF(Indexes_IFAE, Animals, ExpOrder):
         # Freqs = [F for F in set(FreqsB).intersection(FreqsA)]
         #           # if abs(Indexes_IFAE[ExpOrder[1]][Animal][F]) - abs(Indexes_IFAE[ExpOrder[0]][Animal][F]) < -10]
 
-        # MAF_A = [abs(Indexes_IFAE[ExpOrder[1]][Animal][Freq]) -
+        # ExpFIDiff = [abs(Indexes_IFAE[ExpOrder[1]][Animal][Freq]) -
         #           abs(Indexes_IFAE[ExpOrder[0]][Animal][Freq]) for Freq in set(FreqsB).intersection(FreqsA)]
 
         ## Debug
         # print(Animal)
         # print('Before', Indexes_IFAE[ExpOrder[0]][Animal])
         # print('After', Indexes_IFAE[ExpOrder[1]][Animal])
-        # print(MAF_A)
+        # print(ExpFIDiff)
         # print('')
 
-        if not MAF_A: MAF.append(None)
-        else: MAF.append(Freqs[MAF_A.index(min(MAF_A))])
+        if not ExpFIDiff: MAF.append(None)
+        else: MAF.append(Freqs[ExpFIDiff.index(min(ExpFIDiff))])
 
     return(MAF)
 
@@ -205,7 +233,7 @@ def GetMAFIndexes(Indexes_IFAE, MAF, Animals, ExpOrder):
     #            for E, Exp in enumerate(ExpOrder)]
 
     # Override to account for inverted responses
-    # Indexes = [[Analysis.GetNegEquiv(_) for _ in Exp] for Exp in Indexes]
+    # Indexes = [[sAnalysis.GetNegEquiv(_) for _ in Exp] for Exp in Indexes]
     # Indexes = [[_ if _ > 0 else -_ for _ in Exp] for Exp in Indexes]
     # Indexes = [[_ if _ < 100 else _-100 for _ in Exp] for Exp in Indexes]
     # Indexes = [[_ if _ < 100 else 100 for _ in Exp] for Exp in Indexes]
@@ -309,7 +337,7 @@ def OrganizeRecs(Dict, Data, Rate, DataInfo, AnalogTTLs, TimeWindow, Proc=None,
         else: STrial = 'Gap'
 
         if AnalogTTLs:
-            TTLs = Analysis.QuantifyTTLsPerRec(AnalogTTLs, Rec[:,DataInfo['DAqs']['TTLCh']-1])
+            TTLs = sAnalysis.QuantifyTTLsPerRec(AnalogTTLs, Rec[:,DataInfo['DAqs']['TTLCh']-1])
             if len(TTLs) > 1:
                 print('More than one TTL detected!!')
                 # TTLs = [Rec[:,DataInfo['DAqs']['TTLCh']-1].argmax()]
@@ -319,7 +347,7 @@ def OrganizeRecs(Dict, Data, Rate, DataInfo, AnalogTTLs, TimeWindow, Proc=None,
             if not TTLs: print('No TTL detected. Skipping trial...'); continue
 
         else:
-            TTLs = Analysis.QuantifyTTLsPerRec(AnalogTTLs, EventsDict=Events,
+            TTLs = sAnalysis.QuantifyTTLsPerRec(AnalogTTLs, EventsDict=Events,
                                       TTLCh=DataInfo['DAqs']['TTLCh'], Proc=Proc, Rec=R)
 
         if not FilterType:
@@ -327,12 +355,12 @@ def OrganizeRecs(Dict, Data, Rate, DataInfo, AnalogTTLs, TimeWindow, Proc=None,
             else: FilterType = 'bandpass'
 
         if len(DataInfo['DAqs']['RecCh']) == 1:
-            GD = Analysis.FilterSignal(Rec[:,DataInfo['DAqs']['RecCh'][0]-1],
+            GD = sAnalysis.FilterSignal(Rec[:,DataInfo['DAqs']['RecCh'][0]-1],
                                            Rate, FilterFreq, FilterOrder, Filter, FilterType)
             # GD = Rec[:,DataInfo['DAqs']['RecCh'][0]-1]
 
             GD -= GD.mean()
-            GD = Analysis.SliceData(GD, TTLs, -int(TimeWindow[0]*Rate),
+            GD = sAnalysis.SliceData(GD, TTLs, -int(TimeWindow[0]*Rate),
                                      int(TimeWindow[1]*Rate), AnalogTTLs)
 
         elif len(DataInfo['DAqs']['RecCh']) == 3:
@@ -345,15 +373,15 @@ def OrganizeRecs(Dict, Data, Rate, DataInfo, AnalogTTLs, TimeWindow, Proc=None,
             #     np.abs(X-X.mean()),
             #     np.abs(Y-Y.mean()),
             #     np.abs(Z-Z.mean())]
-            # GD = [Analysis.SliceData(
+            # GD = [sAnalysis.SliceData(
             #     _, TTLs, -int(TimeWindow[0]*Rate), int(TimeWindow[1]*Rate),
             #     AnalogTTLs) for _ in GD]
 
-            X = Analysis.FilterSignal(Rec[:,DataInfo['DAqs']['RecCh'][0]-1],
+            X = sAnalysis.FilterSignal(Rec[:,DataInfo['DAqs']['RecCh'][0]-1],
                                           Rate, FilterFreq, FilterOrder, Filter, FilterType)
-            Y = Analysis.FilterSignal(Rec[:,DataInfo['DAqs']['RecCh'][1]-1],
+            Y = sAnalysis.FilterSignal(Rec[:,DataInfo['DAqs']['RecCh'][1]-1],
                                           Rate, FilterFreq, FilterOrder, Filter, FilterType)
-            Z = Analysis.FilterSignal(Rec[:,DataInfo['DAqs']['RecCh'][2]-1],
+            Z = sAnalysis.FilterSignal(Rec[:,DataInfo['DAqs']['RecCh'][2]-1],
                                           Rate, FilterFreq, FilterOrder, Filter, FilterType)
 
             GD = np.mean([
@@ -368,9 +396,9 @@ def OrganizeRecs(Dict, Data, Rate, DataInfo, AnalogTTLs, TimeWindow, Proc=None,
 
             GD -= GD.mean()
 
-            GD = Analysis.SliceData(
+            GD = sAnalysis.SliceData(
                 GD, TTLs, -int(TimeWindow[0]*Rate), int(TimeWindow[1]*Rate),
-                AnalogTTLs)
+            )
 
 
         Dict['IndexTrace'][SFreq][STrial].append(GD)
@@ -399,12 +427,12 @@ def GetAllTrials(
                 for G,Gap in Freq.items():
                     for T,Trial in enumerate(Gap):
                         if Filter:
-                            GD = Analysis.FilterSignal(Trial, Rate, FilterFreq, FilterOrder, Filter, FilterType)
+                            GD = sAnalysis.FilterSignal(Trial, Rate, FilterFreq, FilterOrder, Filter, FilterType)
                         else:
                             GD = Trial.copy()
 
                         GD -= GD.mean()
-                        GD = Analysis.SliceData(GD, GPIASData['TTLs'], -int(TimeWindow[0]*Rate),
+                        GD = sAnalysis.SliceData(GD, GPIASData['TTLs'], -int(TimeWindow[0]*Rate),
                                                  int(TimeWindow[1]*Rate), AnalogTTLs)
 
                         print(Key, F, G, T)
