@@ -7,6 +7,8 @@
 @homepage: https://gitlab.com/Malfatti/SciScripts
 """
 
+print('[Analysis.Plot.Plot] Importing dependencies...')
+
 import os
 import numpy as np
 from itertools import accumulate#, combinations
@@ -16,6 +18,7 @@ from subprocess import check_output
 # from sciscripts.Analysis import Stats
 from sciscripts.Analysis.Analysis import Pairwise
 
+print('[Analysis.Plot.Plot] Done.')
 
 ## Level 0
 def BarAutolabel(Bars, Ax, Color='k', Position='bottom'):
@@ -64,17 +67,6 @@ def FitLegendOutside(Ax, Width=0.8, Loc='center left', Frame=False):
     return(Ax)
 
 
-def GenLegendHandle(color='r', alpha=1, label='', hatch='', marker='line', lw=2):
-    if marker == 'line':
-        Line2D = Return('Line2D')
-        Handler = Line2D([0], [0], color=color, alpha=alpha, lw=lw, label=label)
-    elif marker == 'square':
-        mpatches = Return('mpatches')
-        Handler = mpatches.Patch(color=color, alpha=alpha, label=label, hatch=hatch)
-
-    return(Handler)
-
-
 def GetScreenInfo():
     Screen = {}
     try:
@@ -82,7 +74,8 @@ def GetScreenInfo():
         Screen['DPI'] = str(Screen['DPI']).split('screen #')[1:]
         Screen['DPI'] = [_.split('resolution')[1].split('\\n')[0].split('x')[0].split(' ')[-1] for _ in Screen['DPI']]
         Screen['DPI'] = [int(_) for _ in Screen['DPI']]
-    except FileNotFoundError:
+    except:
+        # Assuming 96 dpi
         Screen['DPI'] = [96]
 
     try:
@@ -91,7 +84,8 @@ def GetScreenInfo():
         Screen['Size_mm'] = [[int(_[:-2]) for _ in f.split(' ') if 'mm' in _] for f in Size ]
         Screen['Size_px'] = [_.split('+')[0].split('x') for f in Size for _ in f.split(' ') if 'x' in _ and '+' in _]
         Screen['Size_px'] = [[int(_) for _ in s] for s in Screen['Size_px']]
-    except FileNotFoundError:
+    except:
+        # Assuming a 15" full HD screen
         Screen['Size_mm'] = [[344, 193]]
         Screen['Size_px'] = [[1920, 1080]]
 
@@ -182,7 +176,7 @@ def Set(Backend='TkCairo', Ax=None, Fig=None, AxArgs={}, FigTitle=None, Params=F
         Params = {
             'backend'             : Backend,
             'image.cmap'          : 'inferno',
-            'savefig.dpi'         : 300,
+            # 'savefig.dpi'         : 300,
             'savefig.format'      : 'svg',
             'savefig.transparent' : True,
 
@@ -196,7 +190,7 @@ def Set(Backend='TkCairo', Ax=None, Fig=None, AxArgs={}, FigTitle=None, Params=F
             'lines.linewidth'     : 1,
 
             'figure.figsize'       : FigSize,
-            'figure.dpi'           : Screen['DPI'][0],
+            # 'figure.dpi'           : Screen['DPI'][0],
             'figure.subplot.wspace': 0.2,
             'figure.subplot.hspace': 0.2,
 
@@ -216,83 +210,120 @@ def Set(Backend='TkCairo', Ax=None, Fig=None, AxArgs={}, FigTitle=None, Params=F
         return(Params)
 
     if Ax:
-        XLim, YLim, ZLim, YTicks = [None]*4
+        ## General
+        Ax.spines['bottom'].set_position(('outward', 2))
+        Ax.spines['left'].set_position(('outward', 2))
+        Ax.spines['right'].set_visible(False)
+        Ax.spines['top'].set_visible(False)
+        Ax.tick_params(top=False, right=False)
+        Ax.patch.set_visible(False)
 
         if 'title' in AxArgs: Ax.set_title(AxArgs['title'])
 
+        ## X
         if 'xlabel' in AxArgs: Ax.set_xlabel(AxArgs['xlabel'])
+        if 'xlim' in AxArgs: Ax.set_xlim(AxArgs['xlim'])
         if 'xticks' in AxArgs: Ax.set_xticks(AxArgs['xticks'])
         if 'xticklabels' in AxArgs: Ax.set_xticklabels(AxArgs['xticklabels'])
-        if 'xlim' in AxArgs: XLim = AxArgs['xlim']
-
-        if 'ylabel' in AxArgs: Ax.set_ylabel(AxArgs['ylabel'])
-        if 'yticklabels' in AxArgs: Ax.set_yticklabels(AxArgs['yticklabels'])
-        if 'ylim' in AxArgs: YLim = AxArgs['ylim']
-        if 'yticks' in AxArgs: YTicks = AxArgs['yticks']
-
-        if 'zlim' in AxArgs: ZLim = AxArgs['zlim']
-        if ZLim:
-            if 'zlabel' in AxArgs: Ax.set_zlabel(AxArgs['zlabel'])
-            if 'zticks' in AxArgs: Ax.set_zticks(AxArgs['zticks'])
-            if 'zticklabels' in AxArgs: Ax.set_zticklabels(AxArgs['zticklabels'])
-            Ax.set_zlim(min(ZLim), max(ZLim))
 
         if 'xtickspacing' in AxArgs:
             import matplotlib.ticker as ticker
             Ax.xaxis.set_major_locator(ticker.MultipleLocator(AxArgs['xtickspacing']))
 
+        if 'xbounds' in AxArgs:
+            Ax.spines['bottom'].set_bounds(AxArgs['xbounds'])
+        else:
+            if len(Ax.get_xticks()) and len(Ax.get_xlim()):
+                Lim, Ticks = Ax.get_xlim(), Ax.get_xticks()
+                Bounds = [Ticks[Ticks >= Lim[0]][0],
+                          Ticks[Ticks <= Lim[1]][-1]]
+                Ax.spines['bottom'].set_bounds(Bounds)
+
+        # XTicks = Ax.get_xticks()
+        # if XLim is None:
+            # XLim = list(Ax.get_xlim())
+            # XInterval = Ax.xaxis.get_data_interval()
+            # Tolerance = (XInterval[1]-XInterval[0])*0.1
+
+            # if len(XTicks) and min(XInterval) > min(XTicks):
+                # XLim[0] = XTicks[XTicks<=min(XInterval)+Tolerance][-1]
+            # if len(XTicks) and max(XInterval) < max(XTicks):
+                # XLim[1] = XTicks[XTicks>=max(XInterval)-Tolerance][0]
+
+        # Ax.set_xlim(XLim)
+
+        # if len(XTicks):
+            # XBound = [XTicks[XTicks >= XLim[0]][0],
+                      # XTicks[XTicks <= XLim[1]][-1]]
+            # Ax.spines['bottom'].set_bounds(XBound[0], XBound[1])
+
+        ## Y
+        if 'ylabel' in AxArgs: Ax.set_ylabel(AxArgs['ylabel'])
+        if 'ylim' in AxArgs: Ax.set_ylim(AxArgs['ylim'])
+        if 'yticks' in AxArgs: Ax.set_yticks(AxArgs['yticks'])
+        if 'yticklabels' in AxArgs: Ax.set_yticklabels(AxArgs['yticklabels'])
+
         if 'ytickspacing' in AxArgs:
             import matplotlib.ticker as ticker
             Ax.yaxis.set_major_locator(ticker.MultipleLocator(AxArgs['ytickspacing']))
 
-        Ax.spines['bottom'].set_position(('outward', 2))
-        Ax.spines['left'].set_position(('outward', 2))
+        if 'ybounds' in AxArgs:
+            Ax.spines['left'].set_bounds(AxArgs['ybounds'])
+        else:
+            if len(Ax.get_yticks()) and len(Ax.get_ylim()):
+                Lim, Ticks = Ax.get_ylim(), Ax.get_yticks()
+                Bounds = [Ticks[Ticks >= Lim[0]][0],
+                          Ticks[Ticks <= Lim[1]][-1]]
+                Ax.spines['left'].set_bounds(Bounds)
 
-        Ax.tick_params(top=False, right=False)
-        Ax.spines['right'].set_visible(False)
-        Ax.spines['top'].set_visible(False)
-        Ax.patch.set_visible(False)
+        # YTicks = Ax.get_yticks()
+        # if YLim is None:
+            # YLim = list(Ax.get_ylim())
+            # YInterval = Ax.yaxis.get_data_interval()
+            # Tolerance = (YInterval[1]-YInterval[0])*0.1
 
-        XTicks = Ax.get_xticks()
-        if XLim is None:
-            XLim = list(Ax.get_xlim())
-            XInterval = Ax.xaxis.get_data_interval()
-            Tolerance = (XInterval[1]-XInterval[0])*0.1
+            # if len(YTicks) and min(YInterval) > min(YTicks):
+                # YLim[0] = YTicks[YTicks<=min(YInterval)+Tolerance][-1]
+            # if len(YTicks) and max(YInterval) < max(YTicks):
+                # YLim[1] = YTicks[YTicks>=max(YInterval)-Tolerance][0]
 
-            if len(XTicks) and min(XInterval) > min(XTicks):
-                XLim[0] = XTicks[XTicks<=min(XInterval)+Tolerance][-1]
-            if len(XTicks) and max(XInterval) < max(XTicks):
-                XLim[1] = XTicks[XTicks>=max(XInterval)-Tolerance][0]
+        # Ax.set_ylim(YLim)
 
-        Ax.set_xlim(XLim)
+        # if len(YTicks):
+            # YBound = [YTicks[YTicks >= YLim[0]][0],
+                      # YTicks[YTicks <= YLim[1]][-1]]
+            # Ax.spines['left'].set_bounds(YBound[0], YBound[1])
 
-        if len(XTicks):
-            XBound = [XTicks[XTicks >= XLim[0]][0],
-                      XTicks[XTicks <= XLim[1]][-1]]
-            Ax.spines['bottom'].set_bounds(XBound[0], XBound[1])
+        # # YInterval = Ax.yaxis.get_data_interval()
+        # # Tolerance = (YInterval[1]-YInterval[0])*0.05
 
-        YInterval = Ax.yaxis.get_data_interval()
-        Tolerance = (YInterval[1]-YInterval[0])*0.05
+        # # if YLim is None:# and YTicks is None:
+            # # YLim = list(Ax.get_ylim())
+            # # YTicks = Ax.get_yticks()
 
-        if YLim is None:# and YTicks is None:
-            YLim = list(Ax.get_ylim())
-            YTicks = Ax.get_yticks()
+            # # if len(YTicks) and min(YInterval) > min(YTicks):
+                # # YLim[0] = YTicks[YTicks<=min(YInterval)+Tolerance][-1]
+            # # if len(YTicks) and max(YInterval) < max(YTicks):
+                # # YLim[1] = YTicks[YTicks>=max(YInterval)-Tolerance][0]
 
-            if len(YTicks) and min(YInterval) > min(YTicks):
-                YLim[0] = YTicks[YTicks<=min(YInterval)+Tolerance][-1]
-            if len(YTicks) and max(YInterval) < max(YTicks):
-                YLim[1] = YTicks[YTicks>=max(YInterval)-Tolerance][0]
+        # # if YTicks is None: YTicks = Ax.get_yticks()
+        # # YTicks = np.array(YTicks)
 
-        if YTicks is None: YTicks = Ax.get_yticks()
-        YTicks = np.array(YTicks)
+        # # if YTicks is not None and len(YTicks):
+            # # YBound = [YTicks[YTicks >= YLim[0]][0],
+                      # # YTicks[YTicks <= YLim[1]][-1]]
+            # # Ax.spines['left'].set_bounds(YBound[0], YBound[1])
+            # # Ax.set_yticks(YTicks)
 
-        if YTicks is not None and len(YTicks):
-            YBound = [YTicks[YTicks >= YLim[0]][0],
-                      YTicks[YTicks <= YLim[1]][-1]]
-            Ax.spines['left'].set_bounds(YBound[0], YBound[1])
-            Ax.set_yticks(YTicks)
+        # # Ax.set_ylim(YLim)
+
+        ## Z
+        if 'zlim' in AxArgs:
+            if 'zlabel' in AxArgs: Ax.set_zlabel(AxArgs['zlabel'])
+            if 'zticks' in AxArgs: Ax.set_zticks(AxArgs['zticks'])
+            if 'zticklabels' in AxArgs: Ax.set_zticklabels(AxArgs['zticklabels'])
+            Ax.set_zlim(min(ZLim), max(ZLim))
 
-        Ax.set_ylim(YLim)
 
     if Fig:
         if FigTitle: Fig.suptitle(FigTitle)
@@ -311,20 +342,28 @@ def Set(Backend='TkCairo', Ax=None, Fig=None, AxArgs={}, FigTitle=None, Params=F
     return(None)
 
 
-def SignificanceBar(X, Y, Text, Ax, FontSize=9, TicksDir='down', lw=1, color='k', AmpF=0):
+def SignificanceBar(X, Y, Text, Ax, TextArgs={}, LineArgs={}, TicksDir='down', LineTextSpacing=None):
     if TicksDir == 'down':
         from matplotlib.markers import TICKDOWN as Tick
-        AmpF = AmpF if AmpF else 1.02
+        AmpF = LineTextSpacing if LineTextSpacing is not None else 1.02
     elif TicksDir == 'up':
         from matplotlib.markers import TICKUP as Tick
-        AmpF = AmpF if AmpF else 0.98
+        AmpF = LineTextSpacing if LineTextSpacing is not None else 0.98
+    elif TicksDir == None:
+        Tick = None
+        AmpF = LineTextSpacing if LineTextSpacing is not None else 1.02
     else:
-        print('TicksDir should be "up" or "down".')
+        print('TicksDir should be "up", "down" or None.')
         return(None)
 
+    TextArgsLoc = {'size': 8, 'ha': 'center', 'va': 'center'}
+    LineArgsLoc = {'color': 'k'}
+    TextArgsLoc = {**TextArgsLoc, **TextArgs}
+    LineArgsLoc = {**LineArgsLoc, **LineArgs}
+
     Yy = max(Y)*AmpF
-    Ax.plot(X, Y, color=color, lw=lw, marker=Tick)
-    Ax.text(sum(X)/2, Yy, Text, fontsize=FontSize, ha='center', va='center')
+    Ax.plot(X, Y, marker=Tick, **LineArgsLoc)
+    Ax.text(sum(X)/2, Yy, Text, **TextArgsLoc)
     return(None)
 
 
@@ -472,7 +511,7 @@ def CheckData(Data, Rate, SpaceAmpF=0.3, TimeWindow=0.2, SkipTime=10, ChLabels=[
     SkipTime = int(Rate*SkipTime/1000)
 
     try:
-        Fig, Ax = plt.subplots(dpi=150, figsize=(7,6))
+        Fig, Ax = plt.subplots(figsize=(7,6))
         for S in range(0,Data.shape[0]-int(TimeWindow*Rate),SkipTime):
             t = np.arange(S, (TimeWindow*Rate)+S, 1)/Rate
 
@@ -513,6 +552,42 @@ def FigAx(Ax, SubPlotsArgs={}):
     return(Fig, Ax, ReturnAx)
 
 
+def GenColorMap(RGB):
+    """
+    Generate colormap based on the provided RGB color.
+
+    Parameters
+    ----------
+    RGB: list
+        List of three values (red, green, blue) representing the desired RGB
+        color. The values should be in the range 0-1 or 0-255.
+
+    Returns
+    -------
+    CMap: colormap
+        A colormap based on the provided RGB color.
+    """
+    ListedColormap = Return('ListedColormap')
+
+    CMap = np.ones((256, 4))
+    for RGBInd,RGBVal in enumerate(RGB):
+        CMap[:, RGBInd] = np.linspace(1, RGBVal, 256)
+    CMap = ListedColormap(CMap)
+
+    return(CMap)
+
+
+def GenLegendHandle(color='r', alpha=1, label='', hatch='', ltype='line', lw=2, marker=None):
+    if ltype == 'line':
+        Line2D = Return('Line2D')
+        Handler = Line2D([0], [0], color=color, alpha=alpha, lw=lw, label=label, marker=marker)
+    elif ltype == 'square':
+        mpatches = Return('mpatches')
+        Handler = mpatches.Patch(color=color, alpha=alpha, label=label, hatch=hatch)
+
+    return(Handler)
+
+
 def GetColors(Key):
     Plt = Return('Plt')
     C = {'colors': ['r', 'g', 'b', 'c', 'm', 'y', 'k']}
@@ -596,8 +671,10 @@ def AllCh(Data, X=[], Colors=[], Labels=[], Leg=[], SpaceAmpF=1, ScaleBar=0, lw=
         Lines[Ch][0].set_color(Colors[Ch])
         if Leg: Lines[Ch][0].set_label(Leg[Ch]); Ax.legend(loc='best')
 
-    AxArgs['yticks'] = YTicks
-    AxArgs['yticklabels'] = Labels if len(Labels) else 1+np.arange(ChNo)
+    # AxArgs['yticks'] = YTicks
+    # AxArgs['yticklabels'] = Labels if len(Labels) else 1+np.arange(ChNo)
+    Ax.set_yticks(YTicks)
+    Ax.set_yticklabels(Labels if len(Labels) else 1+np.arange(ChNo))
     Ax.tick_params(left=False)
     Ax.spines['left'].set_visible(False)
     Ax.spines['left'].set_position(('axes', 0))
