@@ -7,6 +7,8 @@
 @homepage: https://gitlab.com/Malfatti/SciScripts
 """
 
+print('[Analysis.Units.Units] Importing dependencies...')
+
 import os
 import numpy as np
 from glob import glob
@@ -14,7 +16,9 @@ from glob import glob
 from sciscripts.Analysis import Analysis
 from sciscripts.IO import IO
 
-#%%
+print('[Analysis.Units.Units] Done.')
+
+
 ## Level 0
 def FiringRate(SpkTimes, RecLen, Offset=0):
     Spks = SpkTimes-Offset
@@ -146,12 +150,12 @@ def PSTH(Spks, TTLs, Rate, PSTHX, Offset=0):
     return(PSTHX, PSTHY)
 
 
-def PSTHLine(Spks, TTLs, Rate, PSTHX, Offset=0, Output='all'):
+def PSTHLine(Spks, TTLs, PSTHX, Offset=0, Output='all'):
     """ Output: 'all', 'mean', 'norm'"""
 
     PSTHY = np.zeros((len(PSTHX)-1, len(TTLs)), dtype='int32')
     for T, TTL in enumerate(TTLs):
-        Firing = ((Spks-Offset)*1000/Rate) - (TTL*1000/Rate)
+        Firing = (Spks - Offset - TTL)
         Firing = Firing[(Firing >= PSTHX[0]) * (Firing < PSTHX[-1])]
 
         PSTHY[:,T] = np.histogram(Firing, PSTHX)[0]; del(Firing)
@@ -163,6 +167,23 @@ def PSTHLine(Spks, TTLs, Rate, PSTHX, Offset=0, Output='all'):
     else: print('Output should be "all", "mean" or "norm".'); return(None)
 
 
+# def PSTHLine(Spks, TTLs, Rate, PSTHX, Offset=0, Output='all'):
+    # """ Output: 'all', 'mean', 'norm'"""
+#
+    # PSTHY = np.zeros((len(PSTHX)-1, len(TTLs)), dtype='int32')
+    # for T, TTL in enumerate(TTLs):
+        # Firing = ((Spks-Offset)*1000/Rate) - (TTL*1000/Rate)
+        # Firing = Firing[(Firing >= PSTHX[0]) * (Firing < PSTHX[-1])]
+#
+        # PSTHY[:,T] = np.histogram(Firing, PSTHX)[0]; del(Firing)
+#
+    # BinSize = PSTHX[1] - PSTHX[0]
+    # if Output.lower() == 'all': return(PSTHY)
+    # elif Output.lower() == 'mean': return(PSTHY.mean(axis=1))
+    # elif Output.lower() == 'norm': return(PSTHY.sum(axis=1)/(len(TTLs)*BinSize))
+    # else: print('Output should be "all", "mean" or "norm".'); return(None)
+
+
 def Raster(Spks, TTLs, Rate, RasterX, Offset=0):
     RasterY = np.zeros((len(RasterX)-1, len(TTLs)), dtype='float32')
 
@@ -316,7 +337,7 @@ def GetAllUnits(ClustersPath, TimeWindow, ISISize=0.01, TTLCh=None, BinSize=None
                 TTLs = IO.DataLoader(RawRec, Unit='int16', Recording=RawRecdB)[0]
                 if type(TTLs) == dict:
                     _, Proc = IO.OpenEphys.GetChNoAndProcs(RawRec+'/settings.xml')
-                    TTLs = TTLs[Proc][RawRecdB]
+                    TTLs = TTLs[Proc]['0'][RawRecdB]
                 elif type(TTLs) == np.memmap:
                     TTLs = TTLs.reshape((TTLs.shape[0]//ChNoTotal, ChNoTotal))
 
@@ -692,3 +713,37 @@ def FRFreqSharp(CellsParameters, dB):
     FreqBest = np.array([FreqBest_Stim[FreqStim], FreqBest_Stim['Sound_CNO']]).T
 
     return(FR, Sharpness, FreqBest)
+
+## Level 3
+def GetCellsChanges(Cells, Stims, StimRef, AnalysisFile, Save=True, Return=True):
+    CellsChanges = {}
+
+    for Stim in Stims:
+        Ind = (Cells['StimType'] == Stim)
+        CellsChanges[Stim] = Cells['MaxResp'][Ind]
+        CellsChanges[Stim+'-Freqs'] = Cells['BestFreq'][Ind]
+        CellsChanges[Stim+'-dBs'] = Cells['BestdB'][Ind]
+
+    StimPairs = list(combinations(Stims,2))
+
+    for Pair in StimPairs:
+        if StimRef not in Pair: continue
+        if Pair[-1] != StimRef: Pair = (Pair[1], Pair[0])
+
+        Key = 'Vs'.join(Pair)
+        ChangedFreq = CellsChanges[Pair[0]+'-Freqs'] != CellsChanges[Pair[1]+'-Freqs']
+        ChangeddB = CellsChanges[Pair[0]+'-dBs'] != CellsChanges[Pair[1]+'-dBs']
+
+        CellsChanges[Key] = ((CellsChanges[Pair[0]]/CellsChanges[Pair[1]])-1)*100
+        CellsChanges[Key+'-FreqChangePerc'] = len(CellsChanges[Key][ChangedFreq])*100/len(CellsChanges[Key])
+        CellsChanges[Key+'-dBChangePerc'] = len(CellsChanges[Key][ChangeddB])*100/len(CellsChanges[Key])
+
+    if Save: IO.Bin.Write(CellsChanges, AnalysisFile)
+
+    if Return:
+        return(CellsChanges)
+    else:
+        del(CellsChanges)
+        return(None)
+
+
