@@ -88,7 +88,7 @@ def Read(File, ChList=[], Info={}, Verbose=True, AsMMap=True):
             Path = '.'.join(D.split('.')[:-1])
             Path = Path.split(LastCommon)[-1]
             if Verbose: print('Loading', Path, '...')
-            MergeDicts(Data, PathToDict(Path, Read(D, ChList, Info)[0]))
+            MergeDicts(Data, PathToDict(Path, Read(D, ChList, Info, AsMMap=AsMMap)[0]))
 
         if not False in ['_' in _ for _ in Data.keys()]:
             if len(np.unique(['_'.join(_.split('_')[:-1]) for _ in Data.keys()])) == 1:
