@@ -7,10 +7,12 @@
 @homepage: https://gitlab.com/Malfatti/SciScripts
 """
 
+print('[IO.SigGen] Importing dependencies...')
 import numpy as np
 from scipy import signal
 
 from sciscripts.Analysis.Analysis import FilterSignal
+print('[IO.SigGen] Done.')
 
 SoundTTLVal = 0.1; LaserTTLVal = 1
 
@@ -42,8 +44,6 @@ def ApplySoundAmpF(SoundPulseFiltered, Rate, SoundAmpF, NoiseFrequency,
     else:
         Freqs = [str(F) for F in NoiseFrequency]
 
-    # SoundPauseBeforePulse = np.zeros(round(Rate * SoundPauseBeforePulseDur), dtype='float32')
-    # SoundPauseAfterPulse = np.zeros(round(Rate * SoundPauseAfterPulseDur), dtype='float32')
     Start = int(round(SoundPauseBeforePulseDur*Rate))
     End = -int(round(SoundPauseAfterPulseDur*Rate))
     if End == 0: End = TotalDur
@@ -52,17 +52,18 @@ def ApplySoundAmpF(SoundPulseFiltered, Rate, SoundAmpF, NoiseFrequency,
 
     for F,Freq in enumerate(Freqs):
         if Freq not in SoundAmpF:
-            FAmpF = SoundAmpF[list(SoundAmpF)[0]]
-            # IntFreq = [[F,F] for F in SoundAmpF.keys()]
-            # IntFreq = list(map(list, zip(*IntFreq)))
+            # FAmpF = SoundAmpF[list(SoundAmpF)[0]]
+            # Estimate closest calibrated frequency
+            IntFreq = [[F,F] for F in SoundAmpF.keys()]
+            IntFreq = list(map(list, zip(*IntFreq)))
 
-            # IntFreq[1] = [FreqStr2Int(Freq) for Freq in IntFreq[1]]
+            IntFreq[1] = [FreqStr2Int(Freq) for Freq in IntFreq[1]]
 
-            # RealFKey = FreqStr2Int(FKey)
-            # RealFKey = min(IntFreq[1], key=lambda x:abs(x-RealFKey))
-            # RealFKey = IntFreq[0][IntFreq[1].index(RealFKey)]
+            RealFKey = FreqStr2Int(FKey)
+            RealFKey = min(IntFreq[1], key=lambda x:abs(x-RealFKey))
+            RealFKey = IntFreq[0][IntFreq[1].index(RealFKey)]
 
-            # FAmpF = SoundAmpF[RealFKey]
+            FAmpF = SoundAmpF[RealFKey]
             # print('FKey:', RealFKey)
         else:
             FAmpF = SoundAmpF[Freq]
@@ -384,7 +385,6 @@ def SoundLaserStim(Rate, SoundPulseDur, SoundAmpF, NoiseFrequency, System, Laser
     for F in range(SoundLaser.shape[3]):
         for A in range(SoundLaser.shape[2]):
             SoundLaser[:, Map[0]-1, A, F] = SoundTTLUnit+LaserUnit
-            # SoundLaser[:, Map[1]-1, A, F] = SoundUnit[:,A,F]
 
     SoundLaser = np.ascontiguousarray(SoundLaser)
 
