@@ -7,11 +7,13 @@
 @homepage: https://gitlab.com/Malfatti/SciScripts
 """
 
+print('[IO.Wav] Importing dependencies...')
 import numpy as np
 import os
 
 from glob import glob
 from scipy.io import wavfile
+print('[IO.Wav] Done.')
 
 def Read(File):
     Rate, Data = wavfile.read(File)
