@@ -39,10 +39,14 @@ def Root2Dict(El):
         if El.items(): return(dict(El.items()))
         else: return(El.text)
 
+def ReadTree(File):
+    Tree = ElementTree.parse(File)
+    Root = Tree.getroot()
+    return(Tree, Root)
 
+## Level 1
 def Read(File):
-    Tree = ElementTree.parse(File); Root = Tree.getroot()
+    Root = ReadTree(File)[1]
     Info = Root2Dict(Root)
-
     return(Info)
 
